# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from Acquisition import aq_inner

from visaplan.plone.ajaxnavigation.decorators import returns_json

from pdb import set_trace
from visaplan.tools.debug import pp

from visaplan.plone.ajaxnavigation.views import AjaxnavBaseBrowserView
from visaplan.plone.ajaxnavigation.views import EmbedBaseBrowserView


class AjaxnavBrowserView(AjaxnavBaseBrowserView):

    def views_to_try(self, context):
        """
        For folders: call getLayout, and replace the final _view by _embed
        """
        layout = context.getLayout()
        if layout and layout.endswith('_view'):
            liz = layout.split('_')
            yield '_'.join(liz[:-1]+['embed'])
        yield 'folder_content_embed'


class EmbedBrowserView(EmbedBaseBrowserView):

    def children(self):
        context = aq_inner(self.context)
        root = context.absolute_url() + '/'
        children = []
        # no contentFilter for now:
        for o in context.listFolderContents():
            o_id = o.id
            # if necessary: 
            # state = o.restrictedTraverse('@@plone_context_state')
            children.append({
                'href': root + o_id,
                # 'title': state.object_title(),
                'title': o.title,
                })
        return children



# -*- coding: utf-8 -*-
from functools import wraps

from simplejson import dumps


def returns_json(raw):
    """
    Decorate the given function to ...
    - convert the result to JSON, and
    - add the appropriate HTTP headers
    """
    @wraps(raw)
    def wrapped(self, **kwargs):
        dic = raw(self, **kwargs)
        txt = dumps(dic)
        context = self.context
        setHeader = context.REQUEST.RESPONSE.setHeader
        setHeader('Content-Type', 'application/json; charset=utf-8')
        setHeader('Content-Length', len(txt))
        return txt
    return wrapped



# -*- coding: utf-8 -*- vim: ts=8 sts=4 sw=4 si et tw=79
"""\
AJAX navigation support for Plone sites
"""

__author__ = "Tobias Herp <tobias.herp@visaplan.com>"
PROJECTNAME = 'visaplan.plone.ajaxnavigation'



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from visaplan.plone.ajaxnavigation.decorators import returns_json
from zExceptions import Redirect


class SiteInfoView(BrowserView):

    @returns_json
    def __call__(self):
        context = aq_inner(self.context)
        portal_state = context.restrictedTraverse('@@plone_portal_state')
        navroot = portal_state.navigation_root_url()
        url = context.absolute_url()
        if url == navroot:
            return {
                'site_url': navroot,
                }
        else:
            raise Redirect(navroot + '/@@ajax-siteinfo')



# -*- coding: utf-8 -*-
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper

from visaplan.plone.ajaxnavigation.interfaces import (
	IAjaxNavigationClientSettings, IAjaxNavigationInternalSettings,
	)
from plone.z3cform import layout
from z3c.form import form


class AjaxNavigationClientControlPanelForm(RegistryEditForm):
    form.extends(RegistryEditForm)
    schema = IAjaxNavigationClientSettings

AjaxNavigationClientControlPanelView = layout.wrap_form(
		AjaxNavigationClientControlPanelForm, ControlPanelFormWrapper)
AjaxNavigationClientControlPanelView.label = u"AjaxNavigation client settings"


class AjaxNavigationInternalControlPanelForm(RegistryEditForm):
    form.extends(RegistryEditForm)
    schema = IAjaxNavigationInternalSettings

AjaxNavigationInternalControlPanelView = layout.wrap_form(
		AjaxNavigationInternalControlPanelForm, ControlPanelFormWrapper)
AjaxNavigationInternalControlPanelView.label = u"AjaxNavigation internal settings"



# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from Acquisition import aq_inner
from AccessControl.Permissions import view as view_permission
from plone.uuid.interfaces import IUUID
from logging import getLogger

from visaplan.plone.ajaxnavigation.decorators import returns_json
from visaplan.plone.ajaxnavigation.data import PERMISSION_ALIASES

from pdb import set_trace
from visaplan.tools.debug import pp

logger = getLogger('visaplan.plone.ajaxnavigation:views')

__all__ = [  # public interface:
        'AjaxnavBaseBrowserView',  # for @@ajax-nav views
        'EmbedBaseBrowserView',    # for @@embed views
        # MarginInfoBaseBrowserView  (nothing interesting yet)
        ]


class AjaxnavBaseBrowserView(BrowserView):  # ------- [ AjaxnavBaseBV ... [
    """
    BrowserView for @@ajax-nav views:
    these override the __call__ method to return a JSON "object"
    """

    # ------------- [ choose the view for the 'context' JSON key ... [
    def views_to_try(self, context):
        yield 'embed'

    def choose_view(self, context):
        tried = []
        for view_name in self.views_to_try(context):
            if view_name in tried:
                continue
            the_view = getMultiAdapter((context, self.request), name=view_name)
            if the_view:
                return the_view
            tried.append(view_name)
        cls = self.__class__
        if tried:
            logger.error('%(context)r, %(cls)r: no appropriate view found; '
                         'tried %(tried)s', local())
        else:
            logger.error('%(context)r, %(cls)r: no view names!',
                         local())
        return None
    # ------------- ] ... choose the view for the 'context' JSON key ]

    # ---------------------------------- [ construct JSON object ... [
    @returns_json
    def __call__(self):
        """
        Return JSON data for AJAX navigation.

        Return "nothing" if no @@embed view is available.
        Return False if no URL is given.
        """
        context = aq_inner(self.context)
        request = self.request
        form = request.form
        pp(context=context, form=form)
        # set_trace()

        # without a given URL, there probably won't be anything to do for us:
        given_url = form.pop('_given_url', None)
        if (not given_url
                and 0 and 'das sabotiert den Aufruf aus der Adreßzeile'):
            return False

        state = context.restrictedTraverse('@@plone_context_state')
        res = {
            '@title': state.object_title(),
            # for history (like for incoming external links):
            '@url': context.absolute_url() + '/',
            }
        pm = getToolByName(context, 'portal_membership')
        ok = pm.checkPermission(view_permission, context)
        the_view = None
        if ok:
            the_view = self.choose_view(context)
            view_name = None
        elif pm.isAnonymousUser():
            view_name = self.please_login_viewname()
        else:
            view_name = self.insufficient_rights_viewname()

        if the_view is None and view_name is not None:
            pp([('res:', res), ('ok:', ok), ('view_name:', view_name)])
            the_view = getMultiAdapter((context, self.request), name=view_name)

        if the_view is None:
            logger.error('%(context)r: view %(view_name)r not found!',
                         locals())
            res.update({
                '@noajax': True,
                'content': None,
                })
        else:
            try:
                content = the_view()
            except Exception as e:
                logger.error(e)
                logger.exception(e)
                res.update({
                    '@noajax': True,
                    'content': None,
                    })
            else:
                res.update({
                    'content': content,
                    })
        return res
    # ---------------------------------- ] ... construct JSON object ]

    def please_login_viewname(self):
        return 'please_login'

    def insufficient_rights_viewname(self):
        return 'insufficient_rights'
    # -------------------------------- ] ... class AjaxnavBaseBrowserView ]


class EmbedBaseBrowserView(BrowserView):
    """
    Multipurpose base class for @@embed views
    """
    # auszulesen von der perm-Methode:
    interesting_permissions = [
            'edit',  # e.g. to display a pencil
            ]

    def data(self, **kwargs):
        res = self.basedata()
        res.update(self.schemadata(**kwargs))
        return res

    # ----------------------------- [ Bausteine für data() ... [
    def basedata(self):
        context = aq_inner(self.context)
        logger.info('basedata(%(context)r)', locals())
        return {
            'UUID': IUUID(context, None),
            'portal_type': context.portal_type,
            }

    # ------------------ [ EmbedBaseBrowserView.schemadata() ... [
    def schemadata_kwargs(self, accept_unused=False, **kwargs):
        """
        Resolve the keyword arguments to the schemadata method
        and return a dict.

        Keyword options:

        fields -- a whitelist of field names to use
        omit_fields -- a blacklist of field names.

        Of course, only one of both can be specified;
        if none is given, the default is an empty blacklist, i.e. return all.

        By default, unknown options are not accepted.
        To add more options in your own derived class, inject
        accept_unused=True in the kwargs, call this method, and then
        resolve the remaining options in your own method.
        """
        fields = kwargs.pop('fields', [])
        if fields:
            omit_fields = kwargs.pop('omit_fields', None)
            if omit_fields:
                raise TypeError('If fields are given, omit_fields must be empty!'
                                ' (fields=%(fields)r, omit_fields=%(omit_fields)r)'
                                % locals())
            if isinstance(fields, basestring):
                fields = set([fields])
            elif isinstance(fields, set):
                pass
            else:
                fields = set(fields)
        else:
            fields = set()
            omit_fields = kwargs.pop('omit_fields', None) or set()
            if omit_fields is None:
                pass
            elif isinstance(omit_fields, basestring):
                omit_fields = set([omit_fields])
            elif isinstance(omit_fields, set):
                pass
            else:
                omit_fields = set(omit_fields)

        accept_unused = kwargs.get('accept_unused', None)
        if not accept_unused:
            keys = set(kwargs.keys())
            keys.discard('accept_unused')
            if keys:
                keys = sorted(keys)
                raise TypeError('unknown kwarg(s): %(keys)s'
                                ', accept_unused: %(accept_unused)r'
                                % locals())

        return {
            'fields':      fields,
            'omit_fields': omit_fields,
            }

    def schemadata(self, **kwargs):
        """
        Returns the complete schema data, with possible adjustments.

        For the possible keyword options, see the schemadata_kwargs method.
        """
        context = aq_inner(self.context)
        logger.info('schemadata(%(context)r)', locals())
        # pp(context, kwargs)

        resolved_kw = self.schemadata_kwargs(**kwargs)

        # schemadata method is the final consumer of accept_unused:
        accept_unused = kwargs.pop('accept_unused', False)
        if kwargs and not accept_unused:
            raise TypeError('Unused kwarg(s)! (%s)'
                            % ', '.join(kwargs.keys()))

        fields =      resolved_kw.pop('fields')
        omit_fields = resolved_kw.pop('omit_fields')

        use_blacklist = bool(omit_fields)
        use_whitelist = not use_blacklist and bool(fields)

        assert not resolved_kw, (
                'Not all resolved arguments extracted (%(resolved_kw)r)'
                % locals())

        schema = context.Schema()
        keys = schema.keys()
        res = {}
        for key in keys:
            if use_blacklist and key in omit_fields:
                continue
            elif use_whitelist and key not in fields:
                continue
            field = context.getField(key)
            if field is not None:
                val = field.get(context)
                res[key] = val

        return res
    # ------------------ ] ... EmbedBaseBrowserView.schemadata() ]
    # ----------------------------- ] ... Bausteine für data() ]

    def perm(self):
        """
        Return
        Queries all "interesting" permissions, i.e.:
        permissions which are used *in* a view template.
        """
        res = {}
        context = aq_inner(self.context)
        checkperm = getToolByName(context, 'portal_membership').checkPermission
        for alias in self.interesting_permissions:
            perm = PERMISSION_ALIASES[alias]
            if perm not in res:
                res[perm] = checkperm(perm, context)
            if alias != perm:
                res[alias] = res[perm]
        return res

    # ---------- [ "checking permission checker" ... [
    perm_checker_verbose = True
    def perm_checker(self, verbose=None):
        """
        Return a function which checks any permission for the calling context;
        be aware that you are responsible for the correct spelling yourself.

        Usage in template (Python expressions):

        view.perm_checker()('permission string')

        or to check more than one permission, create the checker first, and
        then call it:

        tal:define="check_perm python:view.perm_checker();
                    may_edit   python:check_perm('Modify portal content')"

        """
        context = aq_inner(self.context)
        checkperm = getToolByName(context, 'portal_membership').checkPermission
        if verbose is None:
            verbose = self.perm_checker_verbose
        if verbose:
            known_perms = self.known_permission_strings()
            cls = self.__class__
            def check_verbose(perm):
                if perm not in known_perms:
                    logger.warn('%(cls)s:perm_checker():'
                                ' Unknown permission %(perm)r',
                                {'cls': cls,
                                 'perm': perm,
                                 })
                return checkperm(perm, context)
            return check_verbose
        else:
            def check_simple(perm):
                return checkperm(perm, context)
            return check_simple

    def known_permission_strings(self):
        return frozenset(PERMISSION_ALIASES.values())
    # ---------- ] ... "checking permission checker" ]
    # ---------------------------------- ] ... class EmbedBaseBrowserView ]


class MarginInfoBaseBrowserView(BrowserView):
    """
    A place to add common requirements for BrowserViews
    which are used to feed @@margin-info views
    """



# -*- coding: utf-8 -*-
from visaplan.plone.ajaxnavigation.testing import FUNCTIONAL_TESTING
from visaplan.plone.ajaxnavigation.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getMultiAdapter
from zope.component.interfaces import ComponentLookupError

import unittest


class ViewsIntegrationTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(self.portal, 'Folder', 'other-folder')
        api.content.create(self.portal, 'Collection', 'my-collection')

    def test_ajaxnav_options_is_registered(self):
        view = getMultiAdapter(
            (self.portal['other-folder'], self.portal.REQUEST),
            name='ajaxnav-options'
        )
        self.assertTrue(view(), 'ajaxnav-options is not found')
        self.assertTrue(
            'Sample View' in view(),
            'Sample View is not found in ajaxnav-options'
        )

    def test_ajaxnav_options_in_my_collection(self):
        with self.assertRaises(ComponentLookupError):
            getMultiAdapter(
                (self.portal['my-collection'], self.portal.REQUEST),
                name='ajaxnav-options'
            )


class ViewsFunctionalTest(unittest.TestCase):

    layer = FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])



# -*- coding:utf-8 -*-
from plone.app.registry.browser import controlpanel
from visaplan.plone.ajaxnavigation import _
from visaplan.plone.ajaxnavigation.interfaces import IAjaxNavigationSettings


class AjaxNavigationSettingsEditForm(controlpanel.RegistryEditForm):

    """Control panel edit form."""

    schema = IAjaxNavigationSettings
    label = _(u'AJAX navigation')
    description = _(u'Settings for the visaplan.plone.ajaxnavigation package')


class AjaxNavigationSettingsControlPanel(controlpanel.ControlPanelFormWrapper):

    """Control panel form wrapper."""

    form = AjaxNavigationSettingsEditForm



# -*- coding: utf-8 -*-

from Products.Five.browser import BrowserView

from Products.CMFCore.utils import getToolByName
from plone.registry.interfaces import IRegistry

from visaplan.plone.ajaxnavigation.interfaces import IAjaxNavigationSettings
from visaplan.plone.ajaxnavigation.data import clientside_map
from visaplan.plone.ajaxnavigation.decorators import returns_json


class AjaxnavOptions(BrowserView):

    @returns_json
    def __call__(self):
        """
        Default options for AjaxNav

        (@@ajaxnav-options-default)
        """
        proxy = getToolByName(self.context, 'portal_registry')
        settings = proxy.forInterface(IAjaxNavigationSettings)
        res = {}
        for key, dotted in clientside_map.items():
            res[key] = proxy[dotted]
        return res



# -*- coding: utf-8 -*-
"""Setup testing infrastructure.

For Plone 5 we need to install plone.app.contenttypes.

Tile for collective.cover is only tested in Plone 4.3.
"""
import warnings
with warnings.catch_warnings():
    warnings.simplefilter('ignore', ImportWarning)

    from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
    # from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
    from plone.app.testing import FunctionalTesting
    from plone.app.testing import IntegrationTesting
    from plone.app.testing import PloneSandboxLayer
    from plone.testing import z2

import pkg_resources


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    from plone.app.testing import PLONE_FIXTURE
    HAS_DEXTERITY = False
else:
    from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE as PLONE_FIXTURE
    HAS_DEXTERITY = True

try:
    pkg_resources.get_distribution('collective.cover')
except pkg_resources.DistributionNotFound:
    HAS_COVER = False
else:
    HAS_COVER = True


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        if HAS_COVER:
            import collective.cover
            self.loadZCML(package=collective.cover)

        import plone.app.dexterity
        self.loadZCML(package=plone.app.dexterity)
        import visaplan.js.urlsplit
        self.loadZCML(package=visaplan.js.urlsplit)
        import visaplan.plone.ajaxnavigation
        self.loadZCML(package=visaplan.plone.ajaxnavigation)

    def setUpPloneSite(self, portal):
        if HAS_COVER:
            self.applyProfile(portal, 'collective.cover:default')

        self.applyProfile(portal, 'visaplan.js.urlsplit:default')
        self.applyProfile(portal, 'visaplan.plone.ajaxnavigation:default')


FIXTURE = Fixture()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name='visaplan.plone.ajaxnavigation:Integration',
)
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name='visaplan.plone.ajaxnavigation:Functional',
)


# VISAPLAN_PLONE_AJAXNAVIGATION_ACCEPTANCE_TESTING = FunctionalTesting(
#     bases=(
#         FIXTURE,
#         REMOTE_LIBRARY_BUNDLE_FIXTURE,
#         z2.ZSERVER_FIXTURE,
#     ),
#     name='visaplan.plone.ajaxnavigation:Acceptance',
# )



# -*- coding: utf-8 -*-

def options_dict(**kwargs):
    """
    Create a options dictionary for AjaxNav

    The keys of the dictionary are:

    'whitelist' -- a list of CSS-style selectors to select the objects which
                   will delegate the click handler to their child 'a' objects;
                   default is ['body'].

    'blacklist' -- after applying "delegate", it is possible to "undelegate"
                   certain parts of the DOM;
                   default is the empty list.

    This function returns a dictionary, and the keys of Python dictionaries
    are not guaranteed to follow a certain order.
    Thus, we'll create a little formatting wrapper for reproducibility:
    >>> def tst(**kwargs):
    ...     return sorted(options_dict(**kwargs).items())

    The default is a whitelist of ['body'] and an empty blacklist:
    >>> tst()
    [('blacklist', []), ('whitelist', ['body'])]

    Lists of selectors (currently whitelist and blacklist) can be given as
    strings which will be split whereever commas are found:

    >>> tst(whitelist='one, two')
    [('blacklist', []), ('whitelist', ['one', 'two'])]
    """
    # ./resource/ajaxnavigation.js
    pop = kwargs.pop
    whitelist = pop('whitelist', None) or 'body'
    if isinstance(whitelist, basestring):
        whitelist = [sel.strip() for sel in whitelist.split(',')]
    blacklist = pop('blacklist', [])
    if isinstance(blacklist, basestring):
        blacklist = [sel.strip() for sel in blacklist.split(',')]
    if kwargs:
        raise TypeError('Currently unsupported kwarg(s) %r'
                        % (kwargs.items()[0],
                           ))
    return {
        'whitelist': [sel for sel in whitelist if sel],
        'blacklist': [sel for sel in blacklist if sel],
        }


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import logout
from plone.registry.interfaces import IRegistry
from visaplan.plone.ajaxnavigation.config import PROJECTNAME
from visaplan.plone.ajaxnavigation.interfaces import IAjaxNavigationSettings
from visaplan.plone.ajaxnavigation.testing import INTEGRATION_TESTING
from zope.component import getUtility

import unittest


class ControlPanelTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.controlpanel = self.portal['portal_controlpanel']

    def test_controlpanel_has_view(self):
        view = api.content.get_view(u'ajaxnav-settings', self.portal, self.request)
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@ajaxnav-settings')

    def test_controlpanel_installed(self):
        actions = [
            a.getAction(self)['id'] for a in self.controlpanel.listActions()]
        self.assertIn('ajaxnav', actions)

    def test_controlpanel_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']

        with api.env.adopt_roles(['Manager']):
            qi.uninstallProducts(products=[PROJECTNAME])

        actions = [
            a.getAction(self)['id'] for a in self.controlpanel.listActions()]
        self.assertNotIn('ajaxnav', actions)


class RegistryTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(IAjaxNavigationSettings)

    # -------------------- [ settings for client-side processing ... [
    def test_whitelist_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'whitelist'))
       self.assertEqual(
           self.settings.whitelist, ['body'])

    def test_blacklist_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'blacklist'))
       self.assertEqual(
           self.settings.blacklist, [])

    def test_nested_blacklist_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'nested_blacklist'))
       self.assertFalse(
           self.settings.nested_blacklist)

    def test_view_ids_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'view_ids'))
       self.assertEqual(
           self.settings.view_ids,
           ['view',
            'edit',
            'base_edit',
            ])

    def test_view_prefixes_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'view_prefixes'))
       self.assertEqual(
           self.settings.view_prefixes,
           ['manage_',
            ])

    def test_view_suffixes_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'view_suffixes'))
       self.assertEqual(
           self.settings.view_suffixes,
           ['_view',
            ])

    def test_blacklist_view_ids_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'blacklist_view_ids'))
       self.assertEqual(
           self.settings.blacklist_view_ids,
           ['manage',
            'edit',
            'base_edit',
            ])

    def test_blacklist_view_prefixes_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'blacklist_view_prefixes'))
       self.assertEqual(
           self.settings.blacklist_view_prefixes,
           ['manage_',
            ])

    def test_blacklist_view_suffixes_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'blacklist_view_suffixes'))
       self.assertEqual(
           self.settings.blacklist_view_suffixes,
           ['_edit',
            ])

    def test_selectors_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'selectors'))
       self.assertEqual(
           self.settings.selectors,
           {'content': "#region-content,#content",
            })
    # -------------------- ] ... settings for client-side processing ]

    # ----------------------- [ settings for internal processing ... [
    def test_layout4ajax_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'layout4ajax'))
       self.assertEqual(
           self.settings.layout4ajax,
           { # nothing yet ...
            })

    def test_view4ajax_record_in_registry(self):
       self.assertTrue(hasattr(self.settings, 'view4ajax'))
       self.assertEqual(
           self.settings.view4ajax,
           { # nothing yet ...
            })
    # ----------------------- ] ... settings for internal processing ]

    def test_records_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']

        with api.env.adopt_roles(['Manager']):
            qi.uninstallProducts(products=[PROJECTNAME])

        records = [
            IAjaxNavigationSettings.__identifier__ + '.whitelist',
            IAjaxNavigationSettings.__identifier__ + '.blacklist',
            IAjaxNavigationSettings.__identifier__ + '.nested_blacklist',
            IAjaxNavigationSettings.__identifier__ + '.view_ids',
            IAjaxNavigationSettings.__identifier__ + '.view_prefixes',
            IAjaxNavigationSettings.__identifier__ + '.view_suffixes',
            IAjaxNavigationSettings.__identifier__ + '.blacklist_view_ids',
            IAjaxNavigationSettings.__identifier__ + '.blacklist_view_prefixes',
            IAjaxNavigationSettings.__identifier__ + '.blacklist_view_suffixes',
            IAjaxNavigationSettings.__identifier__ + '.selectors',
            IAjaxNavigationSettings.__identifier__ + '.layout4ajax',
            IAjaxNavigationSettings.__identifier__ + '.view4ajax',
        ]

        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-
# Organisatorisches:
__author__ = """Tobias Herp <tobias.herp@visaplan.com>"""
__docformat__ = 'plaintext'
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer

# Sonstiges (Plone):
from Products.CMFCore.utils import getToolByName
from plone.app.upgrade.utils import loadMigrationProfile

# Unitracc-Tools:
from visaplan.plone.tools.setup import (
        step,
        )

# Logging:
import logging
# ------------------------------------------------------ [ Daten ... [
# UNITRACC_PORTAL_TYPES_1000 = UNITRACC_PORTAL_TYPES[:17]

PROJECTNAME = __package__
PROFILE_ID = PROJECTNAME + ':default'
LOGGER_LABEL = PROJECTNAME + ': setuphandlers'
# ------------------------------------------------------ ] ... Daten ]

logger = logging.getLogger(LOGGER_LABEL)


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            PROJECTNAME + ':uninstall',
        ]


def post_install(context):
    """Post install script"""
    logger.info('Installation complete')

    # Do something at the end of the installation of this package.


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.


# ------------------------ [ Migrationsschritte, ./profiles.zcml ... [
@step
def reload_gs_profile(context, logger=logger):
    loadMigrationProfile(
        context,
        'profile-visaplan.plone.ajaxnavigation:default',
        )
# ------------------------ ] ... Migrationsschritte, ./profiles.zcml ]



# -*- coding: utf-8 -*- vim: et ts=8 sw=4 sts=4 si tw=79 cc=+1
"""Installer for the visaplan.plone.ajaxnavigation package."""

from setuptools import find_packages
from setuptools import setup
from os.path import isfile

package_name = 'visaplan.plone.ajaxnavigation'

# -------------------------------------------- [ get the version ... [
def read_version(fn, sfn):
    main = open(fn).read().strip()
    if sfn is not None and isfile(sfn):
        suffix = valid_suffix(open(sfn).read().strip())
    else:
        suffix = ''
    return main + suffix
    # ... get the version ...
def valid_suffix(suffix):
    """
    Enforce our suffix convention
    """
    suffix = suffix.strip()
    if not suffix:
        return suffix
    allowed = set('.dev0123456789')
    disallowed = set(suffix).difference(allowed)
    if disallowed:
        disallowed = ''.join(sorted(disallowed))
        raise ValueError('Version suffix contains disallowed characters'
                         ' (%(disallowed)s)'
                         % locals())
    chunks = suffix.split('.')
    chunk = chunks.pop(0)
    if chunk:
        raise ValueError('Version suffix must start with "."'
                         ' (%(suffix)r)'
                         % locals())
    if not chunks:
        raise ValueError('Version suffix is too short'
                         ' (%(suffix)r)'
                         % locals())
    for chunk in chunks:
        if not chunk:
            raise ValueError('Empty chunk %(chunk)r in '
                             'version suffix %(suffix)r'
                             % locals())
        char = chunk[0]
        if char in '0123456789':
            raise ValueError('Chunk %(chunk)r of version suffix %(suffix)r'
                             ' starts with a digit'
                             % locals())
        char = chunk[-1]
        if char not in '0123456789':
            raise ValueError('Chunk %(chunk)r of version suffix %(suffix)r'
                             ' doesn\'t end with a digit'
                             % locals())
    return suffix  # ... valid_suffix
    # ... get the version ...
    # ... get the version ...
VERSION = read_version('VERSION',
                       'VERSION_SUFFIX')
# -------------------------------------------- ] ... get the version ]


# ------------------------------------------- [ for setup_kwargs ... [
long_description = '\n\n'.join([
    open('README.rst').read(),
    open('CONTRIBUTORS.rst').read(),
    open('CHANGES.rst').read(),
])

# see as well --> src/visaplan/plone/ajaxnavigation/configure.zcml:
exclude_subpackages = (
        )
exclude_packages = []
for subp in exclude_subpackages:
    exclude_packages.extend([package_name + '.' + subp,
                             package_name + '.' + subp + '.*',
                             ])
packages = find_packages(
            'src',
            exclude=exclude_packages)

def github_urls(package, **kwargs):
    pop = kwargs.pop
    pkg_list = package.split('.')
    res = {}
    readthedocs = pop('readthedocs', False)
    if readthedocs:
        if readthedocs in (1, True):
            readthedocs = ''.join(pkg_list)
        res['Documentation'] = \
            'https://%(readthedocs)s.readthedocs.io' % locals()
        assert 'docs' not in kwargs
    else:
        docs = pop('docs', None)
        if docs is None:
            res['Documentation'] = 'https://pypi.org/project/%(package)s' \
                                   % locals()
        elif docs:
            res['Documentation'] = docs
    if not pop('github', 1):
        assert not kwargs
        return res
    pop_user = pop('pop_user', False)
    if pop_user:
        assert 'pick_user' not in kwargs
        assert 'user' not in kwargs
        user = pkg_list.pop(0)
        package = '.'.join(pkg_list)
    else:
        pick_user = pop('pick_user', 'user' not in kwargs)
        if pick_user:
            user = pkg_list[0]
            if 'user' in kwargs:
                assert pop('user') == user
    base = 'https://github.com/%(user)s/%(package)s' % locals()
    res.update({
        'Source': base,
        'Tracker': base + '/issues',
        })
    return res
project_urls = github_urls(package_name,
                           pop_user=1)  # or pick_user=1, or github=0
# ------------------------------------------- ] ... for setup_kwargs ]

setup_kwargs = dict(
    name=package_name,
    version=VERSION,
    description="Plone add-on to add AJAX navigation",
    long_description=long_description,
    long_description_content_type='text/x-rst',
    # Get more from https://pypi.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Intended Audience :: Developers",
        "Natural Language :: German",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
    ],
    # keywords='Python Plone',
    author='Tobias Herp',
    author_email='tobias.herp@visaplan.com',
    license='GPL version 2+',
    project_urls=project_urls,
    packages=packages,
    namespace_packages=[
        'visaplan',
        'visaplan.plone',
        ],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        # -*- Extra requirements: -*-
        'simplejson',
        'visaplan.js.urlsplit',
        'plone.api',
        # KGS of Plone 4.3.8-9:
        'Products.GenericSetup>=1.8.2',
        # tests failed:
        'plone.app.dexterity',
        'visaplan.plone.tools >= 1.1.5',  # fixed visaplan.kitchen dependency
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            # plone.app.robotframework 1.2.0 requires plone.testing 4.0.11; 
            # plone.app.robotframework 1.3+ drops Plone 4.3 compatibility:
            'plone.testing',
            # currently disabled because of import problems: 
            # 'plone.app.robotframework[debug]',
            'plone.app.robotframework',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.interface import Interface
from plone.supermodel import model
from zope import schema

from visaplan.plone.ajaxnavigation import _


class IVisaplanPloneAjaxnavigationLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IAjaxNavigationSettings(model.Schema):
    """ Schema fields for view "@@ajaxnav-options-default"
    """
    model.fieldset(
            'clientside',
            label=_(u'Clientside processing'),
            fields=[
                'whitelist',
                'blacklist',
                'nested_blacklist',
                'view_ids',
                'view_prefixes',
                'view_suffixes',
                'blacklist_view_ids',
                'blacklist_view_prefixes',
                'blacklist_view_suffixes',
                'selectors',
                ])

    whitelist = schema.List(
        title=_(u"Whitelist"),
        value_type=schema.BytesLine(title=_(u'CSS selector')),
        default=['body',
                 ],
        description=_(
            u'help_whitelist',
            default=u"CSS selectors which activate delegation"
            u" for their a-element descendants"
            ))

    blacklist = schema.List(
        title=_(u"Blacklist"),
        value_type=schema.BytesLine(title=_(u'CSS selector')),
        default=[],
        description=_(
            u'help_blacklist',
            default=u"CSS selectors which deactivate delegation"
            u" for their a-element descendants"
            ))

    nested_blacklist = schema.Bool(
        title=_(u"Nested blacklist"),
        default=False,
        description=_(
            u'help_nested_blacklist',
            default=u"If this is set to True, and if there is a non-empty"
            u" Blacklist, the Blacklist un-delegation will be applied during the"
            u" whitelist processing."
            ))

    view_ids = schema.List(
        title=_(u"View IDs"),
        value_type=schema.BytesLine(title=_(u'view id')),
        default=['view',
                 'edit',
                 'base_edit',
                 ],
        description=_(
            u'help_view_ids',
            default=u"When computing the URLs to try,"
            u" the values given here "
            u' (when following the last slash in the path, of course) '
            u"are considered view ids."
            ))

    view_prefixes = schema.List(
        title=_(u"View prefixes"),
        value_type=schema.BytesLine(title=_(u'view prefix')),
        default=['manage_',
                 ],
        description=_(
            u'help_view_prefixes',
            default=u"If the last path segment of a URL starts with one of"
            " the values given here, it is considered a view name."
            ))

    view_suffixes = schema.List(
        title=_(u"View suffixes"),
        value_type=schema.BytesLine(title=_(u'view suffix')),
        default=['_view',
                 ],
        description=_(
            u'help_view_suffixes',
            default=u"If the last path segment of a URL ends with one of the"
            u" values given here, it is considered a view name."
            ))

    blacklist_view_ids = schema.List(
        title=_(u"Blacklist view IDs"),
        value_type=schema.BytesLine(title=_(u'view id')),
        default=['manage',
                 'edit',
                 'base_edit',
                 ],
        description=_(
            u'help_blacklist_view_ids',
            default=u"When computing the URLs to try, the values given here "
            u'(when following the last slash in the path, of course) '
            u"are considered view ids "
            u"for which an AJAX load won't be tried."
            ))

    blacklist_view_prefixes = schema.List(
        title=_(u"Blacklist view prefixes"),
        value_type=schema.BytesLine(title=_(u'view prefix')),
        default=['manage_',
                 ],
        description=_(
            u'help_blacklist_view_prefixes',
            default=u"If the last path segment of a URL starts with one of the"
            u" values given here, it is considered a view name"
            u" for which an AJAX load won't be tried."
            ))

    blacklist_view_suffixes = schema.List(
        title=_(u"Blacklist view suffixes"),
        value_type=schema.BytesLine(title=_(u'view suffix')),
        default=['_edit',
                 ],
        description=_(
            u'help_blacklist_view_suffixes',
            default=u"If the last path segment of a URL ends with one of the"
            u" values given here, it is considered a view name"
            u" for which an AJAX load won't be tried."
            ))

    selectors = schema.Dict(
        title=_(u"Selectors for data keys"),
        default={'content': "#region-content,#content",
                 },
        key_type=schema.BytesLine(title=_(u'AJAX response key')),
        value_type=schema.BytesLine(title=_(u'CSS selector[,...]')),
        description=_(
            u'help_selectors',
            default=u'For each “normalˮ key from an AJAX response, '
                u'configure a CSS selector which tells where to put the value. '
                u'You can give more than one value, separated by comma, '
                u'which will be tried in order.'
                ))

    model.fieldset(
            'internal',
            label=_(u'Internal processing'),
            fields=[
                'layout4ajax',
                'view4ajax',
                ])

    layout4ajax = schema.Dict(
        title=_(u"Layouts for AJAX"),
        default={},
        key_type=schema.BytesLine(title=_(u'full-page layout')),
        value_type=schema.BytesLine(title=_(u'AJAX view')),
        description=_(
            u'help_layout4ajax',
            default=u'Having detected the layout of the current target '
                u'object, we may map an AJAX version '
                u'to each “full-pageˮ layout.'
                ))

    view4ajax = schema.Dict(
        title=_(u"Views for AJAX"),
        default={},
        key_type=schema.BytesLine(title=_(u'portal_type')),
        value_type=schema.BytesLine(title=_(u'AJAX view')),
        description=_(
            u'help_view4ajax',
            default=u'For each “portal_typeˮ, '
                u'we may map an AJAX version '
                u'corresponding to the standard full-page view.'
                ))



# -*- coding: utf-8 -*-
from Products.CMFCore.permissions import (
        ManagePortal,
        ModifyPortalContent,
        )
from Products.Sessions.SessionPermissions import (
        ACCESS_CONTENTS_PERM as Access_contents_information,
        )
from Products.PlonePAS.permissions import (ManageGroups,
        )
from AccessControl.Permissions import (
        manage_users,             # Manage users
        view as view_permission,  # View
        )

from visaplan.tools.classes import AliasDict

from visaplan.plone.ajaxnavigation.interfaces import IAjaxNavigationSettings

_prefix = IAjaxNavigationSettings.__identifier__ + '.'

clientside_map = {
        key: _prefix + key
        for key in (
                'whitelist',
                'blacklist',
                'nested_blacklist',
                'view_ids',
                'view_prefixes',
                'view_suffixes',
                'blacklist_view_ids',
                'blacklist_view_prefixes',
                'blacklist_view_suffixes',
                'selectors',
                )}

internal_map = {
        key: _prefix + key
        for key in (
                'layout4ajax',
                'view4ajax',
                )}

all_keys = clientside_map.keys() + \
        internal_map.keys()

PERMISSION_ALIASES = AliasDict({
        'view':          view_permission,
        'edit':          ModifyPortalContent,
        'info':          Access_contents_information,
        'manage':        ManagePortal,
        'manage_groups': ManageGroups,
        'manage_users':  manage_users,
        })
